"""This module provides the SingleSignOnAPI class for talking to the
SSO API, plus a few helper classes.
"""

import json

from urllib import quote_plus, urlencode
from piston_mini_client import (
    PistonAPI,
    PistonResponseObject,
    PistonSerializable,
    returns,
    returns_json,
    )
from piston_mini_client.validators import basic_protected, oauth_protected
from piston_mini_client.serializers import serializers


class AccountResponse(PistonResponseObject):
    """A light-weight account object"""
    def __unicode__(self):
        return "Account: %s" % self.displayname


class LazrRestfulSerializer(object):
    """A serializer that renders form-urlencoded json dumps.

    This serializer should be able to serialize arguments so that they can
    be handled by lazr.restful servers.
    """
    def lazr_restfulize(self, key, value):
        return key, (value if key == 'ws.op' else json.dumps(value))
            
    def serialize(self, obj):
        if isinstance(obj, PistonSerializable):
            obj = obj._as_serializable()
        try:
            data = dict(self.lazr_restfulize(k, v) for (k, v) in obj.items())
            return urlencode(data)
        except TypeError:
            raise TypeError("Attempted to serialize invalid object %s" % obj)


class SingleSignOnAPI(PistonAPI):
    """A client for talking to the Ubuntu SSO API.

    If you pass no arguments into the constructor it will try to connect to
    localhost:8000 so you probably want to at least pass in the
    ``service_root`` constructor argument.
    """
    default_service_root = 'http://localhost:8000/api/1.0'
    default_content_type = 'application/x-www-form-urlencoded'
    extra_headers= {'Accept': 'application/json'}
    serializers = {
        'application/x-www-form-urlencoded': LazrRestfulSerializer(),
        }

    @returns_json
    def new_captcha(self):
        """Get a new captcha."""
        data = {'ws.op': 'new'}
        return self._post('captchas', data=data)

    @returns_json
    def register(self, email, password, captcha_id, captcha_solution,
        displayname=None):
        data = {'ws.op': 'register',
            'email': email,
            'password': password,
            'captcha_id': captcha_id,
            'captcha_solution': captcha_solution,
            }
        if displayname is not None:
            data['displayname'] = displayname
        return self._post('registration', data=data)


    @returns_json
    def request_password_reset_token(self, email):
        data = {'ws.op': 'request_password_reset_token',
            'email': email,
            }
        return self._post('registration', data=data)

    @returns_json
    def set_new_password(self, email, new_password, token):
        data = {'ws.op': 'set_new_password',
            'email': email,
            'token': token,
            'new_password': new_password,
            }
        return self._post('registration', data=data)

    @basic_protected
    @returns_json
    def authenticate(self, token_name):
        args = self.json_encode_query({'ws.op': 'authenticate',
            'token_name': token_name,
            })
        return self._get('authentications', args=args)


    @basic_protected
    @returns_json
    def list_tokens(self, consumer_key):
        args = self.json_encode_query({'ws.op': 'list_tokens',
            'consumer_key': consumer_key,
            })
        return self._get('authentications', args=args)

    @basic_protected
    @returns_json
    def validate_token(self, consumer_key, token):
        data = self.json_encode_query({'ws.op': 'validate_token',
            'consumer_key': consumer_key,
            'token': token,
            })
        return self._get('authentications', args=data)

    @basic_protected
    @returns_json
    def invalidate_token(self, consumer_key, token):
        data = {'ws.op': 'invalidate_token',
            'consumer_key': consumer_key,
            'token': token,
            }
        return self._post('authentications', data=data)

    @basic_protected
    @returns_json
    def team_memberships_by_openid(self, openid_identifier, team_names):
        args = self.json_encode_query({'ws.op': 'team_memberships',
            'openid_identifier': openid_identifier,
            'team_names': team_names,
            })
        return self._get('authentications', args=args)

    @basic_protected
    @returns(AccountResponse, none_allowed=True)
    def account_by_email(self, email):
        args = self.json_encode_query({'ws.op': 'account_by_email',
            'email': email,
            })
        return self._get('authentications', args=args)

    @basic_protected
    @returns(AccountResponse, none_allowed=True)
    def account_by_openid(self, openid):
        args = self.json_encode_query({'ws.op': 'account_by_openid',
            'openid': openid,
            })
        return self._get('authentications', args=args)

    @oauth_protected
    @returns(AccountResponse)
    def me(self):
        args = self.json_encode_query({'ws.op': 'me'})
        return self._get('accounts', args=args)

    @oauth_protected
    @returns_json
    def my_team_memberships(self, team_names):
        args = self.json_encode_query({'ws.op': 'team_memberships',
            'team_names': team_names,
            })
        return self._get('accounts', args=args)

    @oauth_protected
    @returns_json
    def validate_email(self, email_token):
        args = self.json_encode_query({'ws.op': 'validate_email',
            'email_token': email_token,
            })
        return self._get('accounts', args=args)

    def json_encode_query(self, data):
        result = {}
        for key, value in data.items():
            if key == 'ws.op':
                result[key] = value
            else:
                result[key] = json.dumps(value)
        return result
